/***************************************************************************/
/***  Name         : build_idor.sql                                      ***/
/***                                                                     ***/
/***  Description  : Build an autonumber from a sequence.                ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports Oracle version 7.3.1 or       ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***                                                                     ***/
/***  Copyright (C) 1999 First Choice Software, Inc.                     ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/* Get the Next ID Number in a Sequence.                                   */
/*                                                                         */
/* Copyright (C) 1999. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE build_id_number (the_seq IN VARCHAR2,
                            auto_num OUT VARCHAR2, ret_int OUT NUMBER)
IS
  new_num NUMBER;
  new_num_str VARCHAR2(30);
  the_format varchar2(80);
  format2 VARCHAR2(80);
  is_done NUMBER;
  the_pos NUMBER;
  pad NUMBER;
  pad_width NUMBER;
  cur_len NUMBER;
  cur_date date;
  date_part varchar2(4);
BEGIN
                                        /* Get the next sequence number    */
                                        /* Get the current date.           */
                                        /* Convert the seq number to a str */
                                        /* Get the format and pad info.    */
  fc_new_numsch (the_seq, new_num);
  cur_date := SYSDATE;
  new_num_str := LTRIM(RTRIM(TO_CHAR(new_num)));

  BEGIN
    SELECT format, padded, value_width INTO the_format, pad, pad_width
    FROM table_num_scheme WHERE name = the_seq;
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      NULL;
  END;

                                        /* If we are padding...            */
                                        /* Get the current length and add  */
                                        /*  as many zeros as we need to at */
                                        /*  front                          */
  IF pad = 1
  THEN
    cur_len := LENGTH(new_num_str);
    WHILE cur_len < pad_width LOOP
      new_num_str := '0' || new_num_str;
      cur_len := cur_len + 1;
    END LOOP;
  END IF;

                                        /* Loop as many times as needed.   */
                                        /* If we find a %i, replace it with*/
                                        /*  the counter.                   */
  is_done := 0;
  WHILE is_done = 0 LOOP
    the_pos := INSTR(the_format, '%i');
    IF the_pos = 0
    THEN
       is_done := 1;
    ELSE
       IF   the_pos > 1
       THEN format2 := SUBSTR(the_format, 1, the_pos - 1);
       END IF;
       IF   format2 IS NULL
       THEN format2 := new_num_str;
       ELSE format2 := format2 || new_num_str;
       END IF;
       IF   the_pos < LENGTH(the_format) - 1
       THEN format2 := format2 || SUBSTR(the_format, the_pos + 2, LENGTH(the_format) - the_pos - 1);
       END IF;
       the_format := format2;
    END IF;
  END LOOP;

                                        /* Loop as many times as needed.   */
                                        /* If we find a %m, replace it with*/
                                        /*  the current month.             */
  date_part := LTRIM(RTRIM(TO_CHAR(cur_date, 'mm')));
  is_done := 0;
  WHILE is_done = 0 LOOP
    the_pos := INSTR(the_format, '%m');
    IF the_pos = 0
    THEN
       is_done := 1;
    ELSE
       IF   the_pos > 1
       THEN format2 := SUBSTR(the_format, 1, the_pos - 1);
       END IF;
       IF   format2 IS NULL
       THEN format2 := date_part;
       ELSE format2 := format2 || date_part;
       END IF;
       IF   the_pos < LENGTH(the_format) - 1
       THEN format2 := format2 || SUBSTR(the_format, the_pos + 2, LENGTH(the_format) - the_pos - 1);
       END IF;
       the_format := format2;
    END IF;
  END LOOP;
                                        /* Loop as many times as needed.   */
                                        /* If we find a %d, replace it with*/
                                        /*  the current day.               */
  date_part := LTRIM(RTRIM(TO_CHAR(cur_date, 'dd')));
  is_done := 0;
  WHILE is_done = 0 LOOP
    the_pos := INSTR(the_format, '%d');
    IF the_pos = 0
    THEN
       is_done := 1;
    ELSE
       IF   the_pos > 1
       THEN format2 := SUBSTR(the_format, 1, the_pos - 1);
       END IF;
       IF   format2 IS NULL
       THEN format2 := date_part;
       ELSE format2 := format2 || date_part;
       END IF;
       IF   the_pos < LENGTH(the_format) - 1
       THEN format2 := format2 || SUBSTR(the_format, the_pos + 2, LENGTH(the_format) - the_pos - 1);
       END IF;
       the_format := format2;
    END IF;
  END LOOP;

                                        /* Loop as many times as needed.   */
                                        /* If we find a %y, replace it with*/
                                        /*  the current year.              */
  date_part := LTRIM(RTRIM(TO_CHAR(cur_date, 'yy')));
  is_done := 0;
  WHILE is_done = 0 LOOP
    the_pos := INSTR(the_format, '%y');
    IF the_pos = 0
    THEN
       is_done := 1;
    ELSE
       IF   the_pos > 1
       THEN format2 := SUBSTR(the_format, 1, the_pos - 1);
       END IF;
       IF   format2 IS NULL
       THEN format2 := date_part;
       ELSE format2 := format2 || date_part;
       END IF;
       IF   the_pos < LENGTH(the_format) - 1
       THEN format2 := format2 || SUBSTR(the_format, the_pos + 2, LENGTH(the_format) - the_pos - 1);
       END IF;
       the_format := format2;
    END IF;
  END LOOP;

                                        /* Loop as many times as needed.   */
                                        /* If we find a %Y, replace it with*/
                                        /*  the current year (four digit). */
  date_part := LTRIM(RTRIM(TO_CHAR(cur_date, 'yyyy')));
  is_done := 0;
  WHILE is_done = 0 LOOP
    the_pos := INSTR(the_format, '%Y');
    IF the_pos = 0
    THEN
       is_done := 1;
    ELSE
       IF   the_pos > 1
       THEN format2 := SUBSTR(the_format, 1, the_pos - 1);
       END IF;
       IF   format2 IS NULL
       THEN format2 := date_part;
       ELSE format2 := format2 || date_part;
       END IF;
       IF   the_pos < LENGTH(the_format) - 1
       THEN format2 := format2 || SUBSTR(the_format, the_pos + 2, LENGTH(the_format) - the_pos - 1);
       END IF;
       the_format := format2;
    END IF;
  END LOOP;

                                        /* Return the format, and ret code */
  auto_num := the_format;
  ret_int := 0;
END;
/

/***************************************************************************/
/* Permission granting statements.                                         */
/*                                                                         */
/* Copyright (C) 1999. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

grant execute on build_id_number to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym build_id_number', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym build_id_number for build_id_number;
