/***************************************************************************/
/***  Name         : check_transor.sql                                   ***/
/***                                                                     ***/
/***  Description  : Validates ClearLogistics transitions.               ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports SQL Server version 6.0 or     ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***                                                                     ***/
/***  Copyright (C) 1997 First Choice Software, Inc.                     ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/* Header declarations.                                                    */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

/***************************************************************************/
/* Check to see if a transition is legal for a given user.                 */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE check_cl_trans (from_cond IN VARCHAR2,
                                            to_cond IN VARCHAR2,
                                            req_type IN VARCHAR2,
                                            user_name IN VARCHAR2,
                                            ret_val OUT NUMBER)
IS
  fc_upper VARCHAR2(80);                /* From condition as upper case.   */
  tc_upper VARCHAR2(80);                /* To condition as upper case.     */
  trans_mask VARCHAR2(255);             /* Transition mask for privclass.  */
  from_status VARCHAR2(80);             /* Default status of from condition*/
  to_status VARCHAR2(80);               /* Default status of to condition. */
  from_stat_objid NUMBER;               /* Default status from cond objid. */
  to_stat_objid NUMBER;                 /* Default status to cond objid.   */
  t_pos NUMBER;                         /* Position for scanning.          */
  rank NUMBER;                          /* Rank of the trans in the mask.  */
  from_cond1 VARCHAR2(80);
  to_cond1 VARCHAR2(80);

BEGIN

                                        /* Convert conditions to upper case*/

  BEGIN
    fc_upper := UPPER(from_cond);
    tc_upper := UPPER(to_cond);

                                        /* Strip off any suffixes.         */
    t_pos := INSTR(fc_upper, '-');
    IF t_pos > 0
    THEN
       from_cond1 := SUBSTR(from_cond, 1, t_pos - 1);
    ELSE
       from_cond1 := from_cond;
    END IF;
    t_pos := INSTR(tc_upper, '-');
    IF t_pos > 0
    THEN
       to_cond1 := SUBSTR(to_cond, 1, t_pos - 1);
    ELSE
       to_cond1 := to_cond;
    END IF;

                                        /* Change open/closed conditions as*/
                                        /*  needed.                        */
    IF fc_upper = 'OPEN'
    THEN
       from_cond1 := 'RQST Open';
    END IF;
    IF fc_upper = 'CLOSED'
    THEN
       from_cond1 := 'RQST Closed';
    END IF;
    IF tc_upper = 'OPEN'
    THEN
       to_cond1 := 'RQST Open';
    END IF;
    IF tc_upper = 'CLOSED'
    THEN
       to_cond1 := 'RQST Closed';
    END IF;

                                        /* Get default status for from     */
                                        /*  from condition. If error, leave*/
    get_list_item (from_cond1, '', from_status, from_stat_objid);
    IF from_status IS NULL OR from_status = ''
    THEN
       ret_val := -1;
       GOTO end_proc;
    END IF;

                                        /* Get default status for to       */
                                        /*  from condition. If error, leave*/
    get_list_item (to_cond1, '', to_status, to_stat_objid);
    IF to_status IS NULL OR to_status = ''
    THEN
       ret_val := -1;
       GOTO end_proc;
    END IF;
  END;

                                        /* Get transition rank. If not     */
                                        /*  found, error and leave.        */

  BEGIN
    SELECT rank INTO rank FROM table_transition
      WHERE focus_type = 49 AND focus_subtype = req_type AND
            from_state2gbst_elm = from_stat_objid AND
            to_state2gbst_elm = to_stat_objid;
   EXCEPTION
    WHEN NO_DATA_FOUND
    THEN
       ret_val := -2;
       GOTO end_proc;
  END;

                                        /* Get trans mask from privclass   */
                                        /*  the user.                      */
  BEGIN
    SELECT trans_mask INTO trans_mask FROM table_privclass 
      WHERE objid = (SELECT user_access2privclass FROM table_user 
                            WHERE login_name = user_name);

                                        /* If user can't transition, error.*/
                                        /* Else, all is OK.                */
    IF SUBSTR(trans_mask, rank + 1, 1) = '0'
    THEN
       ret_val := -3;
    ELSE
       ret_val := 0;
    END IF;
  END;

<<end_proc>>
  NULL;
END;
/

/***************************************************************************/
/* Permission granting statements.                                         */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

grant execute on check_cl_trans to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym check_cl_trans', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym check_cl_trans for check_cl_trans;

/***************************************************************************/
/* Test Driver.                                                            */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE test_proc
IS
  ret_val NUMBER;
BEGIN
/*  check_cl_trans('RQST Open', 'RQST Closed', 'Please Specify', 'sa', ret_val); */
  check_cl_trans('Open', 'Picked', 'Please Specify', 'sa', ret_val);
  dbms_output.put_line(ret_val);
END test_proc;
/
show errors;

grant execute on test_proc to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym test_proc', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym test_proc for test_proc;


