/***************************************************************************/
/***  Name         : numschor.sql                                        ***/
/***                                                                     ***/
/***  Description  : Stored procedure to get the next number in a        ***/
/***                 num scheme sequence. Does it as a unit of work.     ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports SQL Server version 6.0 or     ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***                                                                     ***/
/***  Copyright (C) 1997, 1999 First Choice Software, Inc.               ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/* Header declarations.                                                    */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

/***************************************************************************/
/* Get next number in a number scheme.                                     */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

create or replace procedure fc_new_numsch (num_sch in varchar2,
                                           new_num out number)
IS
BEGIN
  SELECT next_value INTO new_num FROM table_num_scheme WHERE name = num_sch FOR UPDATE;
  UPDATE table_num_scheme SET next_value = next_value + 1 WHERE name = num_sch;
  COMMIT;
END fc_new_numsch;
/
show errors;

/***************************************************************************/
/* Get next number in a part request header sequence.                      */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

create or replace procedure fc_next_pr_seq(hdr_num in varchar2,
                                           new_num out number)
IS
BEGIN
  SELECT sequence_num + 1 INTO new_num FROM table_demand_hdr WHERE header_number = hdr_num FOR UPDATE;
  UPDATE table_demand_hdr SET sequence_num = sequence_num + 1 WHERE header_number = hdr_num;
  COMMIT;
END fc_next_pr_seq;
/
show errors;

/***************************************************************************/
/* Permission granting statements.                                         */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

grant execute on fc_new_numsch to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym fc_new_numsch', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym fc_new_numsch for fc_new_numsch;

grant execute on fc_next_pr_seq to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym fc_next_pr_seq', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym fc_next_pr_seq for fc_next_pr_seq;

/***************************************************************************/
/* Test Driver.                                                            */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE test_proc
IS
  next_number NUMBER;
BEGIN
/*  fc_new_numsch('Case ID', next_number); */
  fc_next_pr_seq('1', next_number);
  dbms_output.put_line(next_number);
END test_proc;
/
show errors;

grant execute on test_proc to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym test_proc', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym test_proc for test_proc;


