/***************************************************************************/
/***  Name         : build_id.sql                                        ***/
/***                                                                     ***/
/***  Description  : Stored procedure to build ID number                 ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports SQL Server version 6.0 or     ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***  Copyright (C) 1998 First Choice Software, Inc.                     ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/***     Header declarations.                                            ***/
/***************************************************************************/

use <db>
go

					/* Drop stored procedures, if      */
					/*  if defined                     */
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'build_id_number')
   DROP PROCEDURE build_id_number
go

/***************************************************************************/
/***     Build an ID number                                              ***/
/***************************************************************************/

CREATE PROCEDURE build_id_number @the_seq VARCHAR(30),
                 @auto_num VARCHAR(80) OUTPUT,
                 @ret_int INT OUTPUT AS
BEGIN
  DECLARE @new_num int,
          @new_num_str varchar(30),
          @format varchar(80),
          @is_done int,
          @the_pos int,
          @pad int,
          @pad_width int,
          @cur_len int,
          @cur_date datetime,
          @date_part varchar(4)

                                        /* Get the next sequence number    */
                                        /* Get the current date.           */
                                        /* Convert the seq number to a str */
                                        /* Get the format and pad info.    */
  EXEC fc_new_numsch @the_seq, @new_num OUT
  SELECT @cur_date = getdate()
  SELECT @new_num_str = LTRIM(RTRIM(CONVERT(char, @new_num)))
  SELECT @format = format, @pad = padded, @pad_width = value_width
    FROM table_num_scheme WHERE name = @the_seq

                                        /* If we are padding...            */
                                        /* Get the current length and add  */
                                        /*  as many zeros as we need to at */
                                        /*  front                          */
  IF @pad = 1
  BEGIN
    SELECT @cur_len = DATALENGTH(@new_num_str)
    WHILE @cur_len < @pad_width
    BEGIN
      SELECT @new_num_str = '0' + @new_num_str
      SELECT @cur_len = @cur_len + 1
    END
  END

                                        /* Loop as many times as needed.   */
                                        /* If we find a %i, replace it with*/
                                        /*  the counter.                   */
  SELECT @is_done = 0
  WHILE @is_done = 0
  BEGIN
    SELECT @the_pos = CHARINDEX('%i', @format)
    IF @the_pos = 0
    BEGIN
       SELECT @is_done = 1
    END
    ELSE
    BEGIN
       SELECT @format = SUBSTRING(@format, 1, @the_pos - 1) + @new_num_str + 
                        SUBSTRING(@format, @the_pos + 2, DATALENGTH(@format) - @the_pos - 1)
    END
  END

                                        /* Loop as many times as needed.   */
                                        /* If we find a %m, replace it with*/
                                        /*  the current month.             */
  SELECT @date_part = LTRIM(RTRIM(CONVERT(char, DATEPART(mm, @cur_date))))
  SELECT @is_done = 0
  WHILE @is_done = 0
  BEGIN
    SELECT @the_pos = CHARINDEX('%m', @format)
    IF @the_pos = 0
    BEGIN
       SELECT @is_done = 1
    END
    ELSE
    BEGIN
       SELECT @format = SUBSTRING(@format, 1, @the_pos - 1) + @date_part + 
                        SUBSTRING(@format, @the_pos + 2, DATALENGTH(@format) - @the_pos - 1)
    END
  END

                                        /* Loop as many times as needed.   */
                                        /* If we find a %d, replace it with*/
                                        /*  the current day.               */
  SELECT @date_part = LTRIM(RTRIM(CONVERT(char, DATEPART(dd, @cur_date))))
  SELECT @is_done = 0
  WHILE @is_done = 0
  BEGIN
    SELECT @the_pos = CHARINDEX('%d', @format)
    IF @the_pos = 0
    BEGIN
       SELECT @is_done = 1
    END
    ELSE
    BEGIN
       SELECT @format = SUBSTRING(@format, 1, @the_pos - 1) + @date_part + 
                        SUBSTRING(@format, @the_pos + 2, DATALENGTH(@format) - @the_pos - 1)
    END
  END

                                        /* Loop as many times as needed.   */
                                        /* If we find a %y, replace it with*/
                                        /*  the current year.              */
  SELECT @date_part = LTRIM(RTRIM(CONVERT(char, DATEPART(yy, @cur_date))))
  SELECT @is_done = 0
  WHILE @is_done = 0
  BEGIN
    SELECT @the_pos = CHARINDEX('%y', @format)
    IF @the_pos = 0
    BEGIN
       SELECT @is_done = 1
    END
    ELSE
    BEGIN
       SELECT @format = SUBSTRING(@format, 1, @the_pos - 1) + @date_part + 
                        SUBSTRING(@format, @the_pos + 2, DATALENGTH(@format) - @the_pos - 1)
    END
  END

                                        /* Return the format, and ret code */
  SELECT @auto_num = @format
  SELECT @ret_int = 0 
END
go

/***************************************************************************/
/***     Permission granting statements.                                 ***/
/***************************************************************************/

GRANT EXECUTE ON build_id_number TO PUBLIC
go

quit